

#define PARSER 1

#include "nest/bird.h"
#include "conf/conf.h"
#include "lib/resource.h"
#include "lib/socket.h"
#include "lib/timer.h"
#include "lib/string.h"
#include "nest/protocol.h"
#include "nest/iface.h"
#include "nest/route.h"
#include "nest/cli.h"
#include "filter/filter.h"

/* FIXME: Turn on YYERROR_VERBOSE and work around lots of bison bugs? */



#include "lib/unix.h"
#include <stdio.h>



#include "lib/krt.h"





#include "nest/rt-dev.h"
#include "nest/password.h"
#include "nest/cmds.h"
#include "lib/lists.h"





#include "proto/bfd/bfd.h"



#include "proto/bgp/bgp.h"



#include "proto/ospf/ospf.h"



#include "proto/pipe/pipe.h"



#include "proto/rip/rip.h"
#include "nest/iface.h"



#include "proto/static/static.h"


static struct keyword keyword_list[] = {
{ "define", DEFINE, NULL },
{ "on", ON, NULL },
{ "off", OFF, NULL },
{ "yes", YES, NULL },
{ "no", NO, NULL },
{ "s", S, NULL },
{ "ms", MS, NULL },
{ "us", US, NULL },
{ "log", LOG, NULL },
{ "syslog", SYSLOG, NULL },
{ "all", ALL, NULL },
{ "debug", DEBUG, NULL },
{ "trace", TRACE, NULL },
{ "info", INFO, NULL },
{ "remote", REMOTE, NULL },
{ "warning", WARNING, NULL },
{ "error", ERROR, NULL },
{ "auth", AUTH, NULL },
{ "fatal", FATAL, NULL },
{ "bug", BUG, NULL },
{ "stderr", STDERR, NULL },
{ "soft", SOFT, NULL },
{ "timeformat", TIMEFORMAT, NULL },
{ "iso", ISO, NULL },
{ "old", OLD, NULL },
{ "short", SHORT, NULL },
{ "long", LONG, NULL },
{ "base", BASE, NULL },
{ "name", NAME, NULL },
{ "confirm", CONFIRM, NULL },
{ "undo", UNDO, NULL },
{ "check", CHECK, NULL },
{ "timeout", TIMEOUT, NULL },
{ "configure", CONFIGURE, NULL },
{ "down", DOWN, NULL },
{ "kernel", KERNEL, NULL },
{ "persist", PERSIST, NULL },
{ "scan", SCAN, NULL },
{ "time", TIME, NULL },
{ "learn", LEARN, NULL },
{ "device", DEVICE, NULL },
{ "routes", ROUTES, NULL },
{ "graceful", GRACEFUL, NULL },
{ "restart", RESTART, NULL },
{ "krt_source", KRT_SOURCE, NULL },
{ "krt_metric", KRT_METRIC, NULL },
{ "table", TABLE, NULL },
{ "krt_prefsrc", KRT_PREFSRC, NULL },
{ "krt_realm", KRT_REALM, NULL },
{ "router", ROUTER, NULL },
{ "id", ID, NULL },
{ "protocol", PROTOCOL, NULL },
{ "template", TEMPLATE, NULL },
{ "preference", PREFERENCE, NULL },
{ "disabled", DISABLED, NULL },
{ "direct", DIRECT, NULL },
{ "interface", INTERFACE, NULL },
{ "import", IMPORT, NULL },
{ "export", EXPORT, NULL },
{ "filter", FILTER, NULL },
{ "none", NONE, NULL },
{ "states", STATES, NULL },
{ "filters", FILTERS, NULL },
{ "receive", RECEIVE, NULL },
{ "limit", LIMIT, NULL },
{ "action", ACTION, NULL },
{ "warn", WARN, NULL },
{ "block", BLOCK, NULL },
{ "disable", DISABLE, NULL },
{ "keep", KEEP, NULL },
{ "filtered", FILTERED, NULL },
{ "password", PASSWORD, NULL },
{ "from", FROM, NULL },
{ "passive", PASSIVE, NULL },
{ "to", TO, NULL },
{ "events", EVENTS, NULL },
{ "packets", PACKETS, NULL },
{ "protocols", PROTOCOLS, NULL },
{ "interfaces", INTERFACES, NULL },
{ "primary", PRIMARY, NULL },
{ "stats", STATS, NULL },
{ "count", COUNT, NULL },
{ "for", FOR, NULL },
{ "commands", COMMANDS, NULL },
{ "preexport", PREEXPORT, NULL },
{ "generate", GENERATE, NULL },
{ "roa", ROA, NULL },
{ "max", MAX, NULL },
{ "flush", FLUSH, NULL },
{ "as", AS, NULL },
{ "listen", LISTEN, NULL },
{ "bgp", BGP, NULL },
{ "v6only", V6ONLY, NULL },
{ "dual", DUAL, NULL },
{ "address", ADDRESS, NULL },
{ "port", PORT, NULL },
{ "passwords", PASSWORDS, NULL },
{ "description", DESCRIPTION, NULL },
{ "sorted", SORTED, NULL },
{ "reload", RELOAD, NULL },
{ "in", IN, NULL },
{ "out", OUT, NULL },
{ "mrtdump", MRTDUMP, NULL },
{ "messages", MESSAGES, NULL },
{ "restrict", RESTRICT, NULL },
{ "memory", MEMORY, NULL },
{ "igp_metric", IGP_METRIC, NULL },
{ "class", CLASS, NULL },
{ "dscp", DSCP, NULL },
{ "wait", WAIT, NULL },
{ "RTS_DUMMY", -((T_ENUM_RTS<<16) | RTS_DUMMY), NULL },
{ "RTS_STATIC", -((T_ENUM_RTS<<16) | RTS_STATIC), NULL },
{ "RTS_INHERIT", -((T_ENUM_RTS<<16) | RTS_INHERIT), NULL },
{ "RTS_DEVICE", -((T_ENUM_RTS<<16) | RTS_DEVICE), NULL },
{ "RTS_STATIC_DEVICE", -((T_ENUM_RTS<<16) | RTS_STATIC_DEVICE), NULL },
{ "RTS_REDIRECT", -((T_ENUM_RTS<<16) | RTS_REDIRECT), NULL },
{ "RTS_RIP", -((T_ENUM_RTS<<16) | RTS_RIP), NULL },
{ "RTS_OSPF", -((T_ENUM_RTS<<16) | RTS_OSPF), NULL },
{ "RTS_OSPF_IA", -((T_ENUM_RTS<<16) | RTS_OSPF_IA), NULL },
{ "RTS_OSPF_EXT1", -((T_ENUM_RTS<<16) | RTS_OSPF_EXT1), NULL },
{ "RTS_OSPF_EXT2", -((T_ENUM_RTS<<16) | RTS_OSPF_EXT2), NULL },
{ "RTS_BGP", -((T_ENUM_RTS<<16) | RTS_BGP), NULL },
{ "RTS_PIPE", -((T_ENUM_RTS<<16) | RTS_PIPE), NULL },
{ "SCOPE_HOST", -((T_ENUM_SCOPE<<16) | SCOPE_HOST), NULL },
{ "SCOPE_LINK", -((T_ENUM_SCOPE<<16) | SCOPE_LINK), NULL },
{ "SCOPE_SITE", -((T_ENUM_SCOPE<<16) | SCOPE_SITE), NULL },
{ "SCOPE_ORGANIZATION", -((T_ENUM_SCOPE<<16) | SCOPE_ORGANIZATION), NULL },
{ "SCOPE_UNIVERSE", -((T_ENUM_SCOPE<<16) | SCOPE_UNIVERSE), NULL },
{ "SCOPE_UNDEFINED", -((T_ENUM_SCOPE<<16) | SCOPE_UNDEFINED), NULL },
{ "RTC_UNICAST", -((T_ENUM_RTC<<16) | RTC_UNICAST), NULL },
{ "RTC_BROADCAST", -((T_ENUM_RTC<<16) | RTC_BROADCAST), NULL },
{ "RTC_MULTICAST", -((T_ENUM_RTC<<16) | RTC_MULTICAST), NULL },
{ "RTC_ANYCAST", -((T_ENUM_RTC<<16) | RTC_ANYCAST), NULL },
{ "RTD_ROUTER", -((T_ENUM_RTD<<16) | RTD_ROUTER), NULL },
{ "RTD_DEVICE", -((T_ENUM_RTD<<16) | RTD_DEVICE), NULL },
{ "RTD_BLACKHOLE", -((T_ENUM_RTD<<16) | RTD_BLACKHOLE), NULL },
{ "RTD_UNREACHABLE", -((T_ENUM_RTD<<16) | RTD_UNREACHABLE), NULL },
{ "RTD_PROHIBIT", -((T_ENUM_RTD<<16) | RTD_PROHIBIT), NULL },
{ "RTD_MULTIPATH", -((T_ENUM_RTD<<16) | RTD_MULTIPATH), NULL },
{ "ROA_UNKNOWN", -((T_ENUM_ROA<<16) | ROA_UNKNOWN), NULL },
{ "ROA_VALID", -((T_ENUM_ROA<<16) | ROA_VALID), NULL },
{ "ROA_INVALID", -((T_ENUM_ROA<<16) | ROA_INVALID), NULL },
{ "show", SHOW, NULL },
{ "status", STATUS, NULL },
{ "summary", SUMMARY, NULL },
{ "route", ROUTE, NULL },
{ "symbols", SYMBOLS, NULL },
{ "add", ADD, NULL },
{ "delete", DELETE, NULL },
{ "dump", DUMP, NULL },
{ "resources", RESOURCES, NULL },
{ "sockets", SOCKETS, NULL },
{ "neighbors", NEIGHBORS, NULL },
{ "attributes", ATTRIBUTES, NULL },
{ "eval", EVAL, NULL },
{ "echo", ECHO, NULL },
{ "enable", ENABLE, NULL },
{ "function", FUNCTION, NULL },
{ "print", PRINT, NULL },
{ "printn", PRINTN, NULL },
{ "unset", UNSET, NULL },
{ "return", RETURN, NULL },
{ "accept", ACCEPT, NULL },
{ "reject", REJECT, NULL },
{ "quitbird", QUITBIRD, NULL },
{ "int", INT, NULL },
{ "bool", BOOL, NULL },
{ "ip", IP, NULL },
{ "prefix", PREFIX, NULL },
{ "pair", PAIR, NULL },
{ "quad", QUAD, NULL },
{ "ec", EC, NULL },
{ "set", SET, NULL },
{ "string", STRING, NULL },
{ "bgpmask", BGPMASK, NULL },
{ "bgppath", BGPPATH, NULL },
{ "clist", CLIST, NULL },
{ "eclist", ECLIST, NULL },
{ "if", IF, NULL },
{ "then", THEN, NULL },
{ "else", ELSE, NULL },
{ "case", CASE, NULL },
{ "true", TRUE, NULL },
{ "false", FALSE, NULL },
{ "rt", RT, NULL },
{ "ro", RO, NULL },
{ "unknown", UNKNOWN, NULL },
{ "generic", GENERIC, NULL },
{ "gw", GW, NULL },
{ "net", NET, NULL },
{ "mask", MASK, NULL },
{ "proto", PROTO, NULL },
{ "source", SOURCE, NULL },
{ "scope", SCOPE, NULL },
{ "cast", CAST, NULL },
{ "dest", DEST, NULL },
{ "ifname", IFNAME, NULL },
{ "ifindex", IFINDEX, NULL },
{ "len", LEN, NULL },
{ "defined", DEFINED, NULL },
{ "contains", CONTAINS, NULL },
{ "reset", RESET, NULL },
{ "prepend", PREPEND, NULL },
{ "first", FIRST, NULL },
{ "last", LAST, NULL },
{ "match", MATCH, NULL },
{ "roa_check", ROA_CHECK, NULL },
{ "empty", EMPTY, NULL },
{ "where", WHERE, NULL },
{ "bfd", BFD, NULL },
{ "min", MIN, NULL },
{ "idle", IDLE, NULL },
{ "rx", RX, NULL },
{ "tx", TX, NULL },
{ "interval", INTERVAL, NULL },
{ "multiplier", MULTIPLIER, NULL },
{ "multihop", MULTIHOP, NULL },
{ "neighbor", NEIGHBOR, NULL },
{ "dev", DEV, NULL },
{ "local", LOCAL, NULL },
{ "sessions", SESSIONS, NULL },
{ "hold", HOLD, NULL },
{ "connect", CONNECT, NULL },
{ "retry", RETRY, NULL },
{ "keepalive", KEEPALIVE, NULL },
{ "startup", STARTUP, NULL },
{ "via", VIA, NULL },
{ "next", NEXT, NULL },
{ "hop", HOP, NULL },
{ "self", SELF, NULL },
{ "default", DEFAULT, NULL },
{ "path", PATH, NULL },
{ "metric", METRIC, NULL },
{ "start", START, NULL },
{ "delay", DELAY, NULL },
{ "forget", FORGET, NULL },
{ "after", AFTER, NULL },
{ "bgp_path", BGP_PATH, NULL },
{ "bgp_local_pref", BGP_LOCAL_PREF, NULL },
{ "bgp_med", BGP_MED, NULL },
{ "bgp_origin", BGP_ORIGIN, NULL },
{ "bgp_next_hop", BGP_NEXT_HOP, NULL },
{ "bgp_atomic_aggr", BGP_ATOMIC_AGGR, NULL },
{ "bgp_aggregator", BGP_AGGREGATOR, NULL },
{ "bgp_community", BGP_COMMUNITY, NULL },
{ "bgp_ext_community", BGP_EXT_COMMUNITY, NULL },
{ "rr", RR, NULL },
{ "rs", RS, NULL },
{ "client", CLIENT, NULL },
{ "cluster", CLUSTER, NULL },
{ "as4", AS4, NULL },
{ "advertise", ADVERTISE, NULL },
{ "ipv4", IPV4, NULL },
{ "capabilities", CAPABILITIES, NULL },
{ "prefer", PREFER, NULL },
{ "older", OLDER, NULL },
{ "missing", MISSING, NULL },
{ "lladdr", LLADDR, NULL },
{ "drop", DROP, NULL },
{ "ignore", IGNORE, NULL },
{ "refresh", REFRESH, NULL },
{ "interpret", INTERPRET, NULL },
{ "communities", COMMUNITIES, NULL },
{ "bgp_originator_id", BGP_ORIGINATOR_ID, NULL },
{ "bgp_cluster_list", BGP_CLUSTER_LIST, NULL },
{ "igp", IGP, NULL },
{ "gateway", GATEWAY, NULL },
{ "recursive", RECURSIVE, NULL },
{ "med", MED, NULL },
{ "ttl", TTL, NULL },
{ "security", SECURITY, NULL },
{ "deterministic", DETERMINISTIC, NULL },
{ "secondary", SECONDARY, NULL },
{ "allow", ALLOW, NULL },
{ "paths", PATHS, NULL },
{ "aware", AWARE, NULL },
{ "ORIGIN_IGP", -((T_ENUM_BGP_ORIGIN<<16) | ORIGIN_IGP), NULL },
{ "ORIGIN_EGP", -((T_ENUM_BGP_ORIGIN<<16) | ORIGIN_EGP), NULL },
{ "ORIGIN_INCOMPLETE", -((T_ENUM_BGP_ORIGIN<<16) | ORIGIN_INCOMPLETE), NULL },
{ "ospf", OSPF, NULL },
{ "area", AREA, NULL },
{ "ospf_metric1", OSPF_METRIC1, NULL },
{ "ospf_metric2", OSPF_METRIC2, NULL },
{ "ospf_tag", OSPF_TAG, NULL },
{ "ospf_router_id", OSPF_ROUTER_ID, NULL },
{ "rfc1583compat", RFC1583COMPAT, NULL },
{ "stub", STUB, NULL },
{ "tick", TICK, NULL },
{ "cost", COST, NULL },
{ "cost2", COST2, NULL },
{ "retransmit", RETRANSMIT, NULL },
{ "hello", HELLO, NULL },
{ "transmit", TRANSMIT, NULL },
{ "priority", PRIORITY, NULL },
{ "dead", DEAD, NULL },
{ "type", TYPE, NULL },
{ "broadcast", BROADCAST, NULL },
{ "bcast", BCAST, NULL },
{ "nonbroadcast", NONBROADCAST, NULL },
{ "nbma", NBMA, NULL },
{ "pointopoint", POINTOPOINT, NULL },
{ "ptp", PTP, NULL },
{ "pointomultipoint", POINTOMULTIPOINT, NULL },
{ "ptmp", PTMP, NULL },
{ "simple", SIMPLE, NULL },
{ "authentication", AUTHENTICATION, NULL },
{ "strict", STRICT, NULL },
{ "cryptographic", CRYPTOGRAPHIC, NULL },
{ "eligible", ELIGIBLE, NULL },
{ "poll", POLL, NULL },
{ "networks", NETWORKS, NULL },
{ "hidden", HIDDEN, NULL },
{ "virtual", VIRTUAL, NULL },
{ "link", LINK, NULL },
{ "only", ONLY, NULL },
{ "buffer", BUFFER, NULL },
{ "large", LARGE, NULL },
{ "normal", NORMAL, NULL },
{ "stubnet", STUBNET, NULL },
{ "tag", TAG, NULL },
{ "external", EXTERNAL, NULL },
{ "lsadb", LSADB, NULL },
{ "ecmp", ECMP, NULL },
{ "weight", WEIGHT, NULL },
{ "nssa", NSSA, NULL },
{ "translator", TRANSLATOR, NULL },
{ "stability", STABILITY, NULL },
{ "global", GLOBAL, NULL },
{ "lsid", LSID, NULL },
{ "instance", INSTANCE, NULL },
{ "real", REAL, NULL },
{ "netmask", NETMASK, NULL },
{ "length", LENGTH, NULL },
{ "merge", MERGE, NULL },
{ "topology", TOPOLOGY, NULL },
{ "state", STATE, NULL },
{ "pipe", PIPE, NULL },
{ "peer", PEER, NULL },
{ "mode", MODE, NULL },
{ "opaque", OPAQUE, NULL },
{ "transparent", TRANSPARENT, NULL },
{ "rip", RIP, NULL },
{ "infinity", INFINITY, NULL },
{ "period", PERIOD, NULL },
{ "garbage", GARBAGE, NULL },
{ "multicast", MULTICAST, NULL },
{ "quiet", QUIET, NULL },
{ "nolisten", NOLISTEN, NULL },
{ "version1", VERSION1, NULL },
{ "plaintext", PLAINTEXT, NULL },
{ "md5", MD5, NULL },
{ "honor", HONOR, NULL },
{ "never", NEVER, NULL },
{ "always", ALWAYS, NULL },
{ "rip_metric", RIP_METRIC, NULL },
{ "rip_tag", RIP_TAG, NULL },
{ "static", STATIC, NULL },
{ "prohibit", PROHIBIT, NULL },
{ "multipath", MULTIPATH, NULL },
{ "blackhole", BLACKHOLE, NULL },
{ "unreachable", UNREACHABLE, NULL },
{ NULL, -1, NULL } };
